#include "StdAfx.h"
#include "XMLDataProvider.h"


//////////////////////////////////////////////////////////////////////////
//DefaultXMLValueConverter
//////////////////////////////////////////////////////////////////////////
class DefaultXMLValueConverter : public IXMLValueConverter
{
public:
	DefaultXMLValueConverter() {}
	~DefaultXMLValueConverter() {}

	CComVariant MakeVariant(LPCTSTR data)
	{
		_ASSERT(data);
		int len = _tcslen(data);
		_ASSERT(len);


		BOOL bNumString = TRUE;
		int dotcount = 0;
		for(int i = 0; i < len; i++)
		{
			if (!((data[i] >= '0' && data[i] <= '9') || data[i] == '.'))
			{
				bNumString = FALSE;
			}
			if (data[i] == '.')
				dotcount++;
		}

		// dot   ڿ Ѵ.
		if (bNumString && dotcount>1)
			bNumString = FALSE;


		// Ͱ ΰ?
		if (bNumString)
		{
			if (_tcschr(data, '.'))
			{
				float val = _wtof(data);
				return CComVariant(val);
			}
			else
			{
				int val = _ttoi(data);
				return CComVariant(val);
			}
		}
		// Ͱ Booleanΰ
		if (_tcsicmp(data, _T("true")) == 0 || _tcsicmp(data, _T("false")) == 0 )
		{
			BOOL bValue = (_tcsicmp(data, _T("true")) == 0);
			return CComVariant(bValue);
		}
		// ڳ Boolean ƴ϶ ڿ 
		return CComVariant(data);
	}
};
//////////////////////////////////////////////////////////////////////////
//XMLDataProvider
//////////////////////////////////////////////////////////////////////////
XMLDataProvider::XMLDataProvider(IDataContext* datacontext)
{
	ATLASSERT(datacontext != NULL);
	m_datacontext = datacontext;
	m_valueConverter = new DefaultXMLValueConverter();
	m_valueValidater = NULL;
}
XMLDataProvider::~XMLDataProvider() 
{
	if (m_valueConverter)
	{
		delete m_valueConverter;
		m_valueConverter = NULL;
	}
}
BOOL XMLDataProvider::IsOneElement(TiXmlElement* element)
{
	ATLASSERT(element != NULL);
	if (element == NULL) return FALSE;

	TiXmlNode* parent = element->Parent();
	if (parent == NULL) return TRUE;

	TiXmlElement* childElement = element->FirstChildElement(element->Value());
	if (childElement)
	{
		childElement = childElement->NextSiblingElement();
		if (childElement)
		{
			return FALSE;
		}
	}
	return TRUE;
}
void XMLDataProvider::LoadAttribute(TiXmlElement* element)
{
	USES_CONVERSION;
	TCHAR szValue[MAX_PATH] = {0,};

	TiXmlAttribute* attr = element->FirstAttribute();
	while (attr)
	{
		XMLUtil::GetAttrValue(element, attr->Name(), szValue, MAX_PATH);
		CString attrstring = A2T(attr->Name());

		if (m_valueValidater)
		{
			if (FALSE == m_valueValidater->IsValid(attrstring, szValue))
			{
				TCHAR szNewValue[MAX_PATH] = {0,};
				m_valueValidater->Fix(attrstring, szValue, szNewValue, MAX_PATH);
				_tcscpy_s(szValue, MAX_PATH, szNewValue);
			}
		}

		CComVariant valueData;
		if (m_valueConverter)
		{
			valueData = m_valueConverter->MakeVariant(szValue);
		}
		else
		{
			valueData = szValue;
		}
		m_datacontext->SetProperty(attrstring, valueData);
		attr = attr->Next();
	}
}
BOOL XMLDataProvider::GetChildValue(TiXmlNode* pChildNode, TCHAR* pszValue, INT nMax)
{
	ZeroMemory(pszValue, nMax);
	USES_CONVERSION;	
	if (pChildNode->Type() == TiXmlNode::COMMENT) return FALSE;
	if (pChildNode->Type() == TiXmlNode::TEXT)
	{
		TiXmlText* pText = pChildNode->ToText();
		if( pText)
		{
			_tcsncpy(pszValue, A2CT(pChildNode->Value()), nMax);			
		}
	}
	if (_tcslen(pszValue) > 0)
	{
		TiXmlNode* pDocChildNode = pChildNode->GetDocument()->FirstChild();
		if (pDocChildNode)
		{
			if (pDocChildNode->Type() == TiXmlNode::DECLARATION)
			{
				TiXmlDeclaration* pDeclarationNode = pDocChildNode->ToDeclaration();
				if (pDeclarationNode)
				{
					LPCSTR pszcEncoding = pDeclarationNode->Encoding();
					if (_stricmp(pszcEncoding, "UTF-8") == 0 ||
						_stricmp(pszcEncoding, "UTF8") == 0)
					{
						wchar_t* pDecodeValue = XMLUtil::Decode_utf8(pChildNode->Value());
						_tcsncpy(pszValue, W2CT(pDecodeValue), nMax);
						delete[] pDecodeValue;
					}
				}
			}
		}
	}
	return (_tcslen(pszValue) > 0);	
}
BOOL XMLDataProvider::GetValue(TiXmlElement* pValueNode, TCHAR* pszValue, INT nMax)
{
	TiXmlNode* pChildNode = pValueNode->FirstChild();
	while (pChildNode)
	{	
		if (pChildNode->Type() == TiXmlNode::COMMENT)
		{
			pChildNode = pChildNode->NextSibling();
			continue;
		}
		return XMLDataProvider::GetChildValue(pChildNode, pszValue, nMax);
	}
	return FALSE;
}
CComVariant XMLDataProvider::LoadChildElement(TiXmlElement* childElement)
{
	TCHAR szValue[MAX_PATH] = {0,};
	TiXmlElement* parent = (TiXmlElement*)childElement->Parent();
	CComVariant valueData;
	if (IsOneElement(childElement) == TRUE && 
		GetValue(childElement, szValue, MAX_PATH))
	{

		if (m_valueValidater)
		{
			USES_CONVERSION;
			CString key = A2T(childElement->Value());
			if (FALSE == m_valueValidater->IsValid(key, szValue))
			{
				TCHAR szNewValue[MAX_PATH] = {0,};
				m_valueValidater->Fix(key, szValue, szNewValue, MAX_PATH);
				_tcscpy_s(szValue, MAX_PATH, szNewValue);
			}
		}

		if (m_valueConverter)
		{
			valueData = m_valueConverter->MakeVariant(szValue);
		}
		else
		{
			valueData = szValue;
		}
	}		
	else
	{
		IDataContext* datacontext = CreateDataContext();
		XMLDataProvider xmlProvider(datacontext);
		if (m_valueValidater)
			xmlProvider.SetValidater(m_valueValidater);
		xmlProvider.Load(childElement);
		
		valueData = PtrToUint(datacontext);
	}
	return valueData;
}
void XMLDataProvider::LoadElement(TiXmlElement* element)
{
	USES_CONVERSION;
	TCHAR szValue[MAX_PATH] = {0,};

	TiXmlElement* childElement = element->FirstChildElement();
	while (childElement)
	{
		CString elementName = A2T(childElement->Value());
		IDataContext* parentNode = m_datacontext;
		CString childName = elementName;

		CString collectionName = elementName;
		collectionName += _T("list");

		//add collection
		IDataContext* collectionNode = m_datacontext->GetChild(collectionName);
		if (collectionNode != NULL)
		{
			parentNode = collectionNode;
			childName.Format(_T("%d"), collectionNode->GetCount());
		}
		else 
		{
			//check collection
			IDataContext* duplicateNode = m_datacontext->GetChild(elementName);
			if (duplicateNode != NULL)
			{
				//make collection node
				collectionNode = CreateDataContext();					
				collectionNode->Add(NULL, duplicateNode);
				
				m_datacontext->SetProperty(collectionName, CComVariant(PtrToUint(collectionNode)));
				m_datacontext->Remove(elementName);

				//add collection
				parentNode = collectionNode;
				childName.Format(_T("%d"), collectionNode->GetCount());
			}
		}
		CComVariant valueData = LoadChildElement(childElement);
		parentNode->SetProperty(childName, valueData);
		
		childElement = childElement->NextSiblingElement();
	}
}
void XMLDataProvider::Load(TiXmlElement* element)
{
	LoadAttribute(element);
	LoadElement(element);	
}/*
void XMLDataProvider::SetData(LPCTSTR key, CComVariant data)
{
	m_datacontext->SetProperty(key, data);
}
CComVariant XMLDataProvider::GetData(LPCTSTR key)
{
	return m_datacontext->GetProperty(key);
}*/